<?php
namespace VamtamElementor\Kits\Kit;

// Extending Elementor's Theme Styles (Kit).

function update_form_field_text_color_control( $controls_manager, $kit ) {
	$prefix       = 'form_field';
	$control_id   = $prefix . '_text_color';
	$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} select,
		{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"],
		{{WRAPPER}} .elementor-select-wrapper,
		{{WRAPPER}} input:not([type="button"]):not([type="submit"])::placeholder,
		{{WRAPPER}} textarea::placeholder,{{WRAPPER}} .elementor-field-textual::placeholder' => reset( $control_data[ 'selectors' ] ),
	];

	$kit->update_control( $control_id, $control_data );
}

function update_form_field_text_focus_color_control( $controls_manager, $kit ) {
	$prefix       = 'form_field_focus';
	$control_id   = $prefix . '_text_color';
	$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} select:focus,
		{{WRAPPER}} .elementor-select-wrapper:focus-within,
		{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]:focus' => reset( $control_data[ 'selectors' ] ),
	];

	$kit->update_control( $control_id, $control_data );
}

function update_form_field_bg_color_control( $controls_manager, $kit ) {
	$prefix       = 'form_field';
	$control_id   = $prefix . '_background_color';
	$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} select,{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]' => 'background-color: {{VALUE}} !important',
	];

	$kit->update_control( $control_id, $control_data );
}

function update_form_field_bg_focus_color_control( $controls_manager, $kit ) {
	$prefix       = 'form_field_focus';
	$control_id   = $prefix . '_background_color';
	$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} select:focus,{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]:focus' => 'background-color: {{VALUE}} !important',
	];

	$kit->update_control( $control_id, $control_data );
}

function update_form_field_border_radius_control( $controls_manager, $kit ) {
	$prefix       = 'form_field';
	$control_id   = $prefix . '_border_radius';
	$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} select,
		{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]' => reset( $control_data[ 'selectors' ] ),
	];

	$kit->update_control( $control_id, $control_data );
}

function update_form_field_border_radius_focus_control( $controls_manager, $kit ) {
	$prefix       = 'form_field_focus';
	$control_id   = $prefix . '_border_radius';
	$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} select:focus,
		{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]:focus' => reset( $control_data[ 'selectors' ] ),
	];

	$kit->update_control( $control_id, $control_data );
}

function update_form_field_transition_duration_focus_control( $controls_manager, $kit ) {
	$prefix       = 'form_field_focus';
	$control_id   = $prefix . '_transition_duration';
	$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} select,
		{{WRAPPER}} .elementor-select-wrapper:before,
		{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]' => reset( $control_data[ 'selectors' ] ),
	];

	$kit->update_control( $control_id, $control_data );
}

function update_form_field_typography_control_group( $controls_manager, $kit ) {
	$control_group = $controls_manager->get_control_groups( 'typography' );

	foreach ( $control_group->get_fields() as $key => $value ) {
	   	$control_id   = 'form_field_typography_' . $key;
		$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

		if ( is_wp_error( $control_data ) ) {
			continue;
		}

		$selector_val = isset( $control_data['selector_value'] ) ? $control_data['selector_value'] : reset( $control_data[ 'selectors' ] );
		$control_data['selectors'] = $control_data['selectors'] + [
			'{{WRAPPER}} select,
			{{WRAPPER}} .select2.select2-container .selection > .select2-selection[role="combobox"]' => $selector_val,
		];

		// Update the control in the stack.
		$kit->update_control( $control_id, $control_data );
   }
}

function update_form_field_border_control_group( $controls_manager, $kit ) {
	$control_group = $controls_manager->get_control_groups( 'border' );
	$prefix        = 'form_field';

	foreach ( $control_group->get_fields() as $key => $value ) {
	   	$control_id   = $prefix . '_border_' . $key;
	   	$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	   	if ( is_wp_error( $control_data ) ) {
		   continue;
	   	}

		if ( $key === 'width' ) {
			update_form_field_border_width_control_responsive( $controls_manager, $kit );
			continue;
		} else {
			$control_data['selectors'] = $control_data['selectors'] + [
				'{{WRAPPER}} select,{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]' => reset( $control_data[ 'selectors' ] ),
			];
		}

	   	// Update the control in the stack.
	   	$kit->update_control( $control_id, $control_data );
   }
}

function update_form_field_border_focus_control_group( $controls_manager, $kit ) {
	$control_group = $controls_manager->get_control_groups( 'border' );
	$prefix        = 'form_field_focus';

	foreach ( $control_group->get_fields() as $key => $value ) {
	   $control_id   = $prefix . '_border_' . $key;
	   $control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	   if ( is_wp_error( $control_data ) ) {
		   continue;
	   }

	   if ( $key === 'width' ) {
			update_form_field_border_width_focus_control_responsive( $controls_manager, $kit );
			continue;
		} else if ( $key === 'color' ) {
			// border-color (focus)
			$control_data['selectors'] = $control_data['selectors'] + [
				'{{WRAPPER}} select:hover,
				{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]:hover,
				{{WRAPPER}} input:hover:not([type="button"]):not([type="submit"]),
				{{WRAPPER}} textarea:hover,
				{{WRAPPER}} .elementor-field-textual:hover' => reset( $control_data[ 'selectors' ] ),
			];
	   } else {
			$control_data['selectors'] = $control_data['selectors'] + [
				'{{WRAPPER}} select:focus,
				{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]:focus' => reset( $control_data[ 'selectors' ] ),
			];
		}

	   // Update the control in the stack.
	   $kit->update_control( $control_id, $control_data );
   }
}

function update_form_field_box_shadow_control_group( $controls_manager, $kit ) {
	$control_group = $controls_manager->get_control_groups( 'box-shadow' );
	$prefix        = 'form_field';

	$control_id    = $prefix . '_box_shadow_box_shadow';
	$control_data  = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} select,{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]' => reset( $control_data[ 'selectors' ] ),
	];

	// Update the control in the stack.
	$kit->update_control( $control_id, $control_data );
}

function update_form_field_box_shadow_focus_control_group( $controls_manager, $kit ) {
	$control_group = $controls_manager->get_control_groups( 'box-shadow' );
	$prefix        = 'form_field_focus';

	$control_id    = $prefix . '_box_shadow_box_shadow';
	$control_data  = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} select:focus,{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]:focus' => reset( $control_data[ 'selectors' ] ),
	];

	// Update the control in the stack.
	$kit->update_control( $control_id, $control_data );
}

function update_form_field_border_width_control_responsive( $controls_manager, $kit ) {
	$control_id = 'form_field_border_width';

	foreach ( [ $control_id, $control_id . '_tablet', $control_id . '_mobile' ] as $device_cid  ) {
		$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $device_cid );

		if ( is_wp_error( $control_data ) ) {
			continue;
		}

		$control_data['selectors'] = $control_data['selectors'] + [
			'{{WRAPPER}} select,
			{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]' => reset( $control_data[ 'selectors' ] ),
		];

		$kit->update_control( $device_cid, $control_data );
	}
}

function update_form_field_border_width_focus_control_responsive( $controls_manager, $kit ) {
	$control_id = 'form_field_focus_border_width';

	foreach ( [ $control_id, $control_id . '_tablet', $control_id . '_mobile' ] as $device_cid  ) {
		$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $device_cid );

		if ( is_wp_error( $control_data ) ) {
			continue;
		}

		$control_data['selectors'] = $control_data['selectors'] + [
			'{{WRAPPER}} select:focus,
			{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]:focus' => reset( $control_data[ 'selectors' ] ),
		];

		$kit->update_control( $device_cid, $control_data );
	}
}

function update_form_field_padding_control_responsive( $controls_manager, $kit ) {
	$control_id   = 'form_field_padding';

	foreach ( [ $control_id, $control_id . '_tablet', $control_id . '_mobile' ] as $device_cid  ) {
		$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $device_cid );

		if ( is_wp_error( $control_data ) ) {
			continue;
		}

		$control_data['selectors'] = $control_data['selectors'] + [
			'{{WRAPPER}} select,{{WRAPPER}} .select2.select2-container .select2-selection[role="combobox"]' => reset( $control_data[ 'selectors' ] ),
		];

		$kit->update_control( $device_cid, $control_data );
	}
}

// Form Fields section
add_action( 'elementor/element/kit/section_form_fields/before_section_end', function( $kit, $args ) {
	$controls_manager = \Elementor\Plugin::instance()->controls_manager;
	update_form_field_typography_control_group( $controls_manager, $kit );

	update_form_field_text_color_control( $controls_manager, $kit );
	update_form_field_text_focus_color_control( $controls_manager, $kit );

	update_form_field_bg_color_control( $controls_manager, $kit );
	update_form_field_bg_focus_color_control( $controls_manager, $kit );

	update_form_field_border_radius_control( $controls_manager, $kit );
	update_form_field_border_radius_focus_control( $controls_manager, $kit );

	update_form_field_transition_duration_focus_control( $controls_manager, $kit );

	update_form_field_border_control_group( $controls_manager, $kit );
	update_form_field_border_focus_control_group( $controls_manager, $kit );

	update_form_field_box_shadow_control_group( $controls_manager, $kit );
	update_form_field_box_shadow_focus_control_group( $controls_manager, $kit );

	update_form_field_padding_control_responsive( $controls_manager, $kit );
}, 10, 2 );

function update_headings_typography_control_groups( $controls_manager, $kit ) {
	$control_group = $controls_manager->get_control_groups( 'typography' );

	// Headings 1-6.
	for ( $i = 1; $i < 7; $i++ ) {
		// For each typography field.
		foreach ( $control_group->get_fields() as $key => $value ) {
			$control_id   = 'h' . $i . '_typography_' . $key;
			// For each device.
			foreach ( [ $control_id, $control_id . '_tablet', $control_id . '_mobile' ] as $device_cid  ) {
				$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $device_cid );

				if ( is_wp_error( $control_data ) ) {
					continue;
				}

				$selector_val = isset( $control_data['selector_value'] ) ? $control_data['selector_value'] : reset( $control_data[ 'selectors' ] );
				$control_data['selectors'] = $control_data['selectors'] + [
					"{{WRAPPER}} .font-h{$i}" => $selector_val,
				];

				// Update the control in the stack.
				$kit->update_control( $device_cid, $control_data );
			}
		}
	}
}

function update_headings_colors( $controls_manager, $kit ) {
	$control_group = $controls_manager->get_control_groups( 'typography' );

	// Headings 1-6.
	for ( $i = 1; $i < 7; $i++ ) {
		$control_id   = "h{$i}_color";
		$control_data = $controls_manager->get_control_from_stack( $kit->get_name() . '-' . $kit->get_main_id(), $control_id );

		if ( is_wp_error( $control_data ) ) {
			continue;
		}

		$selector_val = isset( $control_data['selector_value'] ) ? $control_data['selector_value'] : reset( $control_data[ 'selectors' ] );
		$control_data['selectors'] = $control_data['selectors'] + [
			"{{WRAPPER}} .font-h{$i}" => $selector_val,
		];

		// Update the control in the stack.
		$kit->update_control( $control_id, $control_data );
	}
}

// Typography section
add_action( 'elementor/element/kit/section_typography/before_section_end', function( $kit, $args ) {
	$controls_manager = \Elementor\Plugin::instance()->controls_manager;
	update_headings_typography_control_groups( $controls_manager, $kit );
	update_headings_colors( $controls_manager, $kit );
}, 10, 2 );
