<?php
namespace VamtamElementor\Widgets\MenuCart;

// Extending the Menu Cart widget.

// Sadly there's no update_group_controls() function.
// We have to update each control in the group manually.
function update_product_title_typography_control_group( $controls_manager, $widget ) {
	$control_group = $controls_manager->get_control_groups( 'typography' );

	foreach ( $control_group->get_fields() as $key => $value ) {
	   $control_id   = 'product_title_typography_' . $key;
	   $control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	   if ( is_wp_error( $control_data ) ) {
		   continue;
	   }

	   $selector_val = isset( $control_data['selector_value'] ) ? $control_data['selector_value'] : reset( $control_data[ 'selectors' ] );
	   if ( $key === 'font_size' ) {
		   // Font-size should apply to product-remove & qty select.
		   $control_data['selectors'] = $control_data['selectors'] + [
			   '{{WRAPPER}} .vamtam-elementor-menu-cart__header > .item-count, {{WRAPPER}} .product-remove svg.vamtam-close, {{WRAPPER}} .product-price .vamtam-quantity > select' => $selector_val
		   ];
	   } else {
		   // All font options should apply to item-count.
		   $control_data['selectors'] = $control_data['selectors'] + [
			   '{{WRAPPER}} .vamtam-elementor-menu-cart__header > .item-count' => $selector_val
		   ];
	   }

	   // Update the control in the stack.
	   $widget->update_control( $control_id, $control_data );
   }
}

function update_product_title_color_control( $controls_manager, $widget ) {
	$control_id   = 'product_title_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	// Title color should apply to product-remove & qty select.
	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .product-remove svg.vamtam-close, {{WRAPPER}} .product-price .quantity .vamtam-quantity > select, {{WRAPPER}} .product-price .quantity .vamtam-quantity > svg' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

// Products Section
add_action( 'elementor/element/woocommerce-menu-cart/section_product_tabs_style/before_section_end', function( $widget, $args ) {
	$controls_manager = \Elementor\Plugin::instance()->controls_manager;
	update_product_title_typography_control_group( $controls_manager, $widget );
	update_product_title_color_control( $controls_manager, $widget );
}, 10, 2 );

function update_checkout_button_bg_color_control( $controls_manager, $widget ) {
	$control_id   = 'checkout_button_background_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	// Title color should apply to product-remove & qty select.
	$control_data['selectors'] = $control_data['selectors'] + [
		'.woocommerce.woocommerce-cart .cart_totals a.checkout-button, .woocommerce.woocommerce-cart input[name="apply_coupon"]' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_checkout_button_color_control( $controls_manager, $widget ) {
	$control_id   = 'checkout_button_text_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	// Title color should apply to product-remove & qty select.
	$control_data['selectors'] = $control_data['selectors'] + [
		'.woocommerce.woocommerce-cart .cart_totals a.checkout-button, .woocommerce-cart .coupon input[name="apply_coupon"]' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

// Sadly there's no update_group_controls() function.
// We have to update each control in the group manually.
function update_checkout_border_control_group( $controls_manager, $widget ) {
	$control_group = $controls_manager->get_control_groups( 'border' );

	foreach ( $control_group->get_fields() as $key => $value ) {
	   $control_id   = 'checkout_border_' . $key;
	   $control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	   if ( is_wp_error( $control_data ) ) {
		   continue;
	   }

		// border-style / border-width / border-color
		$control_data['selectors'] = $control_data['selectors'] + [
			'.woocommerce.woocommerce-cart .cart_totals a.checkout-button, .woocommerce-cart .coupon input[name="apply_coupon"]' => reset( $control_data[ 'selectors' ] ),
		];


	   // Update the control in the stack.
	   $widget->update_control( $control_id, $control_data );
   }
}

// Buttons section
add_action( 'elementor/element/woocommerce-menu-cart/section_style_buttons/before_section_end', function( $widget, $args ) {
	$controls_manager = \Elementor\Plugin::instance()->controls_manager;
	update_checkout_button_color_control( $controls_manager, $widget );
	update_checkout_button_bg_color_control( $controls_manager, $widget );
	update_checkout_border_control_group( $controls_manager, $widget );
}, 10, 2 );

// Responsive control
function update_divider_gap_control_responsive( $controls_manager, $widget ) {
	$control_id   = 'divider_gap';

	foreach ( [ $control_id, $control_id . '_tablet', $control_id . '_mobile' ] as $device_cid  ) {
		$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $device_cid );

		if ( is_wp_error( $control_data ) ) {
			continue;
		}

		$control_data['selectors'] = [
			'{{WRAPPER}} .elementor-menu-cart__product:not(:last-of-type)' => 'margin-bottom: {{SIZE}}{{UNIT}}',
			'{{WRAPPER}} .elementor-menu-cart__product:not(:first-of-type)' => 'margin-top: {{SIZE}}{{UNIT}}',
		];

		$widget->update_control( $device_cid, $control_data );
	}
}

function update_divider_width_control( $controls_manager, $widget ) {
	$control_id   = 'divider_width';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = [
		'{{WRAPPER}} .elementor-menu-cart__subtotal' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

// Products section
add_action( 'elementor/element/woocommerce-menu-cart/section_product_tabs_style/before_section_end', function( $widget, $args ) {
	$controls_manager = \Elementor\Plugin::instance()->controls_manager;
	update_divider_gap_control_responsive( $controls_manager, $widget );
	update_divider_width_control( $controls_manager, $widget );
}, 10, 2 );
