<?php
namespace VamtamElementor\Widgets\NavMenu;

// Extending the Nav Menu widget.

function update_color_menu_item_active_color_control( $controls_manager, $widget ) {
	$control_id   = 'color_menu_item_active';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .elementor-nav-menu--main .current-menu-ancestor > .elementor-item' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_pointer_color_menu_item_active_color_control( $controls_manager, $widget ) {
	$control_id   = 'pointer_color_menu_item_active';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .elementor-nav-menu--main:not(.e--pointer-framed) .current-menu-ancestor > .elementor-item:before,
		{{WRAPPER}} .elementor-nav-menu--main:not(.e--pointer-framed) .current-menu-ancestor > .elementor-item:after' => 'background-color: {{VALUE}}',
		'{{WRAPPER}} .e--pointer-framed .current-menu-ancestor > .elementor-item:before,
		{{WRAPPER}} .e--pointer-framed .current-menu-ancestor > .elementor-item:after' => 'border-color: {{VALUE}}',
	];

	$widget->update_control( $control_id, $control_data );
}

function get_nav_menu_top_level_items_count( $menu_name ) {
	$menu_items      = wp_get_nav_menu_items( $menu_name );
	$top_level_items = 0;

	foreach ( $menu_items as $key => $menu_item ) {
		if ( $menu_item->menu_item_parent != 0 ) {
			continue;
		} else {
			++$top_level_items;
		}
	}

	return $top_level_items;
}

function add_individual_nav_items_color_controls( $controls_manager, $widget ) {

	$widget->add_control(
		'individual_nav_items_colors',
		[
			'label' => __( 'Individual Nav Items Colors', 'vamtam-elementor-integration' ),
			'type' => $controls_manager::SWITCHER,
			'description' => __( 'Use different colors for each of the menu items. These colors have priority over the general ones.', 'vamtam-elementor-integration' ),
			'return_value' => 'yes',
		]
	);

	$widget->start_controls_tabs( 'individual_nav_items_colors_tabs' );

	$widget->start_controls_tab(
		'tab_individual_nav_items_colors',
		[
			'label' => __( 'Normal', 'vamtam-elementor-integration' ),
			'condition' => [
				'individual_nav_items_colors' => 'yes',
			],
		]

	);

	// Get menu items (top-level).
	$top_level_items_count = 7; // until we find a solution to the getSettings() problem: https://github.com/elementor/elementor/issues/8698

	for ( $i = 0; $i < $top_level_items_count; $i++ ) {
		$widget->add_control(
			"nav_item_{$i}_color_label",
			[
				'type' => $controls_manager::RAW_HTML,
				'raw' => sprintf( __( 'Item %d', 'vamtam-elementor-integration' ), $i + 1 ),
				'content_classes' => 'elementor-control-field-title',
				'condition' => [
					'individual_nav_items_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"nav_item_{$i}_text_color",
			[
				'label' => __( 'Text Color', 'vamtam-elementor-integration' ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item' => 'color: {{VALUE}}',
				],
				'condition' => [
					'individual_nav_items_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"nav_item_{$i}_color_hr",
			[
				'type' => $controls_manager::DIVIDER,
				'condition' => [
					'individual_nav_items_colors' => 'yes',
				],
			]
		);
	}

	$widget->end_controls_tab();

	$widget->start_controls_tab(
		'tab_individual_nav_items_colors_hover',
		[
			'label' => __( 'Hover', 'vamtam-elementor-integration' ),
			'condition' => [
				'individual_nav_items_colors' => 'yes',
			],
			'description' => __( 'Use different colors for each of the menu items.', 'vamtam-elementor-integration' ),

		]
	);

	for ( $i = 0; $i < $top_level_items_count; $i++ ) {
		$widget->add_control(
			"nav_item_{$i}_color_hover_label",
			[
				'type' => $controls_manager::RAW_HTML,
				'raw' => sprintf( __( 'Item %d', 'vamtam-elementor-integration' ), $i + 1 ),
				'content_classes' => 'elementor-control-field-title',
				'condition' => [
					'individual_nav_items_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"nav_item_{$i}_text_color_hover",
			[
				'label' => __( 'Text Color', 'vamtam-elementor-integration' ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item:hover,
					{{WRAPPER}} .elementor-nav-menu--main  > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item.elementor-item-active,
					{{WRAPPER}} .elementor-nav-menu--main  > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item.highlighted,
					{{WRAPPER}} .elementor-nav-menu--main  > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item:focus' => 'color: {{VALUE}}',
				],
				'condition' => [
					'individual_nav_items_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"nav_item_{$i}_pointer_color_hover",
			[
				'label' => sprintf( __( 'Pointer Color', 'vamtam-elementor-integration' ), $i + 1 ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main:not(.e--pointer-framed)  > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item:before,
					{{WRAPPER}} .elementor-nav-menu--main:not(.e--pointer-framed) > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item:after' => 'background-color: {{VALUE}}',
					'{{WRAPPER}} .e--pointer-framed > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item:before,
					{{WRAPPER}} .e--pointer-framed > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item:after' => 'border-color: {{VALUE}}',
				],
				'condition' => [
					'individual_nav_items_colors' => 'yes',
					'pointer!' => [ 'none', 'text' ],
				],
			]
		);
		$widget->add_control(
			"nav_item_{$i}_color_hover_hr",
			[
				'type' => $controls_manager::DIVIDER,
				'condition' => [
					'individual_nav_items_colors' => 'yes',
				],
			]
		);
	}

	$widget->end_controls_tab();

	$widget->start_controls_tab(
		'tab_individual_nav_items_colors_active',
		[
			'label' => __( 'Active', 'vamtam-elementor-integration' ),
			'condition' => [
				'individual_nav_items_colors' => 'yes',
			],
		]
	);

	for ( $i = 0; $i < $top_level_items_count; $i++ ) {
		$widget->add_control(
			"nav_item_{$i}_color_active_label",
			[
				'type' => $controls_manager::RAW_HTML,
				'raw' => sprintf( __( 'Item %d', 'vamtam-elementor-integration' ), $i + 1 ),
				'content_classes' => 'elementor-control-field-title',
				'condition' => [
					'individual_nav_items_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"nav_item_{$i}_text_color_active",
			[
				'label' => __( 'Text Color', 'vamtam-elementor-integration' ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item.elementor-item-active,
					{{WRAPPER}} .elementor-nav-menu--main > .elementor-nav-menu > .current-menu-ancestor.menu-item:nth-child(' . ( $i + 1 ) . ') > .elementor-item' => 'color: {{VALUE}}',
				],
				'condition' => [
					'individual_nav_items_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"nav_item_{$i}_pointer_color_active",
			[
				'label' => sprintf( __( 'Pointer Color', 'vamtam-elementor-integration' ), $i + 1 ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main:not(.e--pointer-framed) > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item.elementor-item-active:before,
					{{WRAPPER}} .elementor-nav-menu--main:not(.e--pointer-framed)  > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item.elementor-item-active:after' => 'background-color: {{VALUE}}',
					'{{WRAPPER}} .e--pointer-framed > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item.elementor-item-active:before,
					{{WRAPPER}} .e--pointer-framed  > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-item.elementor-item-active:after' => 'border-color: {{VALUE}}',
					'{{WRAPPER}} .elementor-nav-menu--main:not(.e--pointer-framed) > .elementor-nav-menu > .current-menu-ancestor.menu-item:nth-child(' . ( $i + 1 ) . ') > .elementor-item:before,
					{{WRAPPER}} .elementor-nav-menu--main:not(.e--pointer-framed) > .elementor-nav-menu > .current-menu-ancestor.menu-item:nth-child(' . ( $i + 1 ) . ') > .elementor-item:after' => 'background-color: {{VALUE}}',
					'{{WRAPPER}} .e--pointer-framed > .elementor-nav-menu > .current-menu-ancestor.menu-item:nth-child(' . ( $i + 1 ) . ') > .elementor-item:before,
					{{WRAPPER}} .e--pointer-framed > .elementor-nav-menu > .current-menu-ancestor.menu-item:nth-child(' . ( $i + 1 ) . ') > .elementor-item:after' => 'border-color: {{VALUE}}',
				],
				'condition' => [
					'individual_nav_items_colors' => 'yes',
					'pointer!' => [ 'none', 'text' ],
				],
			]
		);
		$widget->add_control(
			"nav_item_{$i}_color_active_hr",
			[
				'type' => $controls_manager::DIVIDER,
				'condition' => [
					'individual_nav_items_colors' => 'yes',
				],
			]
		);
	}

	$widget->end_controls_tab();

	$widget->end_controls_tabs();
}

function add_individual_dropdown_color_controls( $controls_manager, $widget ) {

	$widget->add_control(
		'individual_dropdown_colors',
		[
			'label' => __( 'Individual Dropdown Colors', 'vamtam-elementor-integration' ),
			'type' => $controls_manager::SWITCHER,
			'description' => __( 'Use different colors for each of the dropdowns. These colors have priority over the general ones.', 'vamtam-elementor-integration' ),
			'return_value' => 'yes',
		]
	);

	$widget->start_controls_tabs( 'individual_dropdown_colors_tabs' );

	$widget->start_controls_tab(
		'tab_individual_dropdown_colors',
		[
			'label' => __( 'Normal', 'vamtam-elementor-integration' ),
			'condition' => [
				'individual_dropdown_colors' => 'yes',
			],
		]

	);

	// Get menu items (top-level).
	// $top_level_items_count = get_nav_menu_top_level_items_count( $menu_name );
	$top_level_items_count = 7; // until we find a solution to the getSettings() problem: https://github.com/elementor/elementor/issues/8698

	for ( $i = 0; $i < $top_level_items_count; $i++ ) {
		$widget->add_control(
			"dropdown_{$i}_color_label",
			[
				'type' => $controls_manager::RAW_HTML,
				'raw' => sprintf( __( 'Item %d', 'vamtam-elementor-integration' ), $i + 1 ),
				'content_classes' => 'elementor-control-field-title',
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"dropdown_{$i}_text_color",
			[
				'label' => __( 'Text Color', 'vamtam-elementor-integration' ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-nav-menu--dropdown a,' => 'color: {{VALUE}}',
				],
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"dropdown_{$i}_bg_color",
			[
				'label' => sprintf( __( 'Background Color', 'vamtam-elementor-integration' ), $i + 1 ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-nav-menu--dropdown' => 'background-color: {{VALUE}}',
				],
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"dropdown_{$i}_color_hr",
			[
				'type' => $controls_manager::DIVIDER,
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
	}

	$widget->end_controls_tab();

	$widget->start_controls_tab(
		'tab_individual_dropdown_colors_hover',
		[
			'label' => __( 'Hover', 'vamtam-elementor-integration' ),
			'condition' => [
				'individual_dropdown_colors' => 'yes',
			],
			'description' => __( 'Use different colors for each of the menu items.', 'vamtam-elementor-integration' ),

		]
	);

	for ( $i = 0; $i < $top_level_items_count; $i++ ) {
		$widget->add_control(
			"dropdown_{$i}_color_hover_label",
			[
				'type' => $controls_manager::RAW_HTML,
				'raw' => sprintf( __( 'Item %d', 'vamtam-elementor-integration' ), $i + 1 ),
				'content_classes' => 'elementor-control-field-title',
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"dropdown_{$i}_text_color_hover",
			[
				'label' => __( 'Text Color', 'vamtam-elementor-integration' ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-nav-menu--dropdown a:hover,
					{{WRAPPER}}  .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-nav-menu--dropdown a.elementor-item-active,
					{{WRAPPER}}  .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-nav-menu--dropdown a.highlighted' => 'color: {{VALUE}}',
				],
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"dropdown_{$i}_bg_color_hover",
			[
				'label' => sprintf( __( 'Background Color', 'vamtam-elementor-integration' ), $i + 1 ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-nav-menu--dropdown a:hover,
					{{WRAPPER}}  .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-nav-menu--dropdown a.elementor-item-active,
					{{WRAPPER}}  .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-nav-menu--dropdown a.highlighted' => 'background-color: {{VALUE}}',
				],
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"dropdown_{$i}_color_hover_hr",
			[
				'type' => $controls_manager::DIVIDER,
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
	}

	$widget->end_controls_tab();

	$widget->start_controls_tab(
		'tab_individual_dropdown_colors_active',
		[
			'label' => __( 'Active', 'vamtam-elementor-integration' ),
			'condition' => [
				'individual_dropdown_colors' => 'yes',
			],
		]
	);

	for ( $i = 0; $i < $top_level_items_count; $i++ ) {
		$widget->add_control(
			"dropdown_{$i}_color_active_label",
			[
				'type' => $controls_manager::RAW_HTML,
				'raw' => sprintf( __( 'Item %d', 'vamtam-elementor-integration' ), $i + 1 ),
				'content_classes' => 'elementor-control-field-title',
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"dropdown_{$i}_text_color_active",
			[
				'label' => __( 'Text Color', 'vamtam-elementor-integration' ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-nav-menu--dropdown a.elementor-item-active' => 'color: {{VALUE}}',
				],
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"dropdown_{$i}_bg_color_active",
			[
				'label' => sprintf( __( 'Background Color', 'vamtam-elementor-integration' ), $i + 1 ),
				'type' => $controls_manager::COLOR,
				'default' => '',
				'selectors' => [
					'{{WRAPPER}} .elementor-nav-menu--main > .elementor-nav-menu > .menu-item:nth-child(' . ( $i + 1 ) . ') .elementor-nav-menu--dropdown a.elementor-item-active' => 'background-color: {{VALUE}}',
				],
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
		$widget->add_control(
			"dropdown_{$i}_color_active_hr",
			[
				'type' => $controls_manager::DIVIDER,
				'condition' => [
					'individual_dropdown_colors' => 'yes',
				],
			]
		);
	}

	$widget->end_controls_tab();

	$widget->end_controls_tabs();
}

function update_pointer_width_control( $controls_manager, $widget ) {
	$control_id   = 'pointer_width';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .e--pointer-framed .current-menu-ancestor > a.elementor-item:before' => 'border-width: {{SIZE}}{{UNIT}} !important',
		'{{WRAPPER}} .e--pointer-framed.e--animation-draw .current-menu-ancestor > .elementor-item:before' => 'border-width: 0 0 {{SIZE}}{{UNIT}} {{SIZE}}{{UNIT}} !important',
		'{{WRAPPER}} .e--pointer-framed.e--animation-draw .current-menu-ancestor > .elementor-item:after' => 'border-width: {{SIZE}}{{UNIT}} {{SIZE}}{{UNIT}} 0 0 !important',
		'{{WRAPPER}} .e--pointer-framed.e--animation-corners .current-menu-ancestor > .elementor-item:before' => 'border-width: {{SIZE}}{{UNIT}} 0 0 {{SIZE}}{{UNIT}} !important',
		'{{WRAPPER}} .e--pointer-framed.e--animation-corners .current-menu-ancestor > .elementor-item:after' => 'border-width: 0 {{SIZE}}{{UNIT}} {{SIZE}}{{UNIT}} 0 !important',
		'{{WRAPPER}} .e--pointer-underline .current-menu-ancestor > .elementor-item:after,
			{{WRAPPER}} .e--pointer-overline .current-menu-ancestor > .elementor-item:before,
			{{WRAPPER}} .e--pointer-double-line .current-menu-ancestor > .elementor-item:before,
			{{WRAPPER}} .e--pointer-double-line .current-menu-ancestor > .elementor-item:after' => 'height: {{SIZE}}{{UNIT}} !important',
	];

	$widget->update_control( $control_id, $control_data );
}

// Stye - Main Menu section
add_action( 'elementor/element/nav-menu/section_style_main-menu/before_section_end', function( $widget, $args ) {
	$controls_manager = \Elementor\Plugin::instance()->controls_manager;
	update_color_menu_item_active_color_control( $controls_manager, $widget );
	update_pointer_color_menu_item_active_color_control( $controls_manager, $widget );
	update_pointer_width_control( $controls_manager, $widget );
	add_individual_nav_items_color_controls( $controls_manager, $widget );
}, 10, 2 );

// Style - Dropdown section
add_action( 'elementor/element/nav-menu/section_style_dropdown/before_section_end', function( $widget, $args ) {
	$controls_manager = \Elementor\Plugin::instance()->controls_manager;
	add_individual_dropdown_color_controls( $controls_manager, $widget );
}, 10, 2 );
