<?php
namespace VamtamElementor\Widgets\ProductsBase;

// Extending the Products_Base widget class (related/archives/normal products widgets).

// Responsive control
function update_image_spacing_control_responsive( $controls_manager, $widget ) {
	$control_id = 'image_spacing';

	foreach ( [ $control_id, $control_id . '_tablet', $control_id . '_mobile' ] as $device_cid  ) {
		$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $device_cid );

		if ( is_wp_error( $control_data ) ) {
			continue;
		}

		$control_data['selectors'] = [
			'{{WRAPPER}} ul.products li.product .vamtam-product-content > .woocommerce-loop-product__title' => 'padding-top: {{SIZE}}{{UNIT}}',
		];

		$widget->update_control( $device_cid, $control_data );
	}
}

function update_button_bg_color_control( $controls_manager, $widget ) {
	$control_id   = 'button_background_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}}.elementor-wc-products .added_to_cart' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_button_hover_bg_color_control( $controls_manager, $widget ) {
	$control_id   = 'button_hover_background_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}}.elementor-wc-products .added_to_cart:hover' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_button_color_control( $controls_manager, $widget ) {
	$control_id   = 'button_text_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}}.elementor-wc-products .added_to_cart' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_button_hover_color_control( $controls_manager, $widget ) {
	$control_id   = 'button_hover_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}}.elementor-wc-products .added_to_cart:hover' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_button_hover_border_color_control( $controls_manager, $widget ) {
	$control_id   = 'button_hover_border_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}}.elementor-wc-products .added_to_cart:hover' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_button_text_padding_control( $controls_manager, $widget ) {
	$control_id   = 'button_text_padding';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}}.elementor-wc-products .added_to_cart' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_button_border_radius_control( $controls_manager, $widget ) {
	$control_id   = 'button_border_radius';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}}.elementor-wc-products .added_to_cart' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

// Responsive control
function update_button_spacing_control_responsive( $controls_manager, $widget ) {
	$control_id   = 'button_spacing';

	foreach ( [ $control_id, $control_id . '_tablet', $control_id . '_mobile' ] as $device_cid  ) {
		$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $device_cid );

		if ( is_wp_error( $control_data ) ) {
			continue;
		}

		$control_data['selectors'] = [
			'{{WRAPPER}} ul.products li.product .vamtam-product-content' => 'margin-bottom: {{SIZE}}{{UNIT}}',
		];

		$widget->update_control( $device_cid, $control_data );
	}
}

// Sadly there's no update_group_controls() function.
// We have to update each control in the group manually.
function update_button_border_control_group( $controls_manager, $widget ) {
	$control_group = $controls_manager->get_control_groups( 'border' );

	foreach ( $control_group->get_fields() as $key => $value ) {
	   $control_id   = 'button_border_' . $key;
	   $control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	   if ( is_wp_error( $control_data ) ) {
		   continue;
	   }

		// border-style / border-width / border-color
		$control_data['selectors'] = $control_data['selectors'] + [
			'{{WRAPPER}}.elementor-wc-products .added_to_cart' => reset( $control_data[ 'selectors' ] ),
		];


	   // Update the control in the stack.
	   $widget->update_control( $control_id, $control_data );
   }
}

// Sadly there's no update_group_controls() function.
// We have to update each control in the group manually.
function update_button_typography_control_group( $controls_manager, $widget ) {
	$control_group = $controls_manager->get_control_groups( 'typography' );

	foreach ( $control_group->get_fields() as $key => $value ) {
	   $control_id   = 'button_typography_' . $key;
	   $control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	   if ( is_wp_error( $control_data ) ) {
		   continue;
	   }

	   $selector_val = isset( $control_data['selector_value'] ) ? $control_data['selector_value'] : reset( $control_data[ 'selectors' ] );
	   $control_data['selectors'] = $control_data['selectors'] + [
			'{{WRAPPER}}.elementor-wc-products .added_to_cart' => $selector_val,
		];

	   // Update the control in the stack.
	   $widget->update_control( $control_id, $control_data );
   }
}

function update_products_base_controls( $widget, $args ) {
	$controls_manager = \Elementor\Plugin::instance()->controls_manager;
	update_image_spacing_control_responsive( $controls_manager, $widget );
	update_button_color_control( $controls_manager, $widget );
	update_button_hover_color_control( $controls_manager, $widget );
	update_button_bg_color_control( $controls_manager, $widget );
	update_button_hover_bg_color_control( $controls_manager, $widget );
	update_button_hover_border_color_control( $controls_manager, $widget );
	update_button_border_control_group( $controls_manager, $widget );
	update_button_typography_control_group( $controls_manager, $widget );
	update_button_text_padding_control( $controls_manager, $widget );
	update_button_border_radius_control( $controls_manager, $widget );
	update_button_spacing_control_responsive( $controls_manager, $widget );
}

// Products Button section (add_to_cart, view_cart).
add_action( 'elementor/element/woocommerce-product-related/section_products_style/before_section_end', 'VamtamElementor\Widgets\ProductsBase\update_products_base_controls', 10, 2 );
add_action( 'elementor/element/wc-archive-products/section_products_style/before_section_end', 'VamtamElementor\Widgets\ProductsBase\update_products_base_controls', 10, 2 );
add_action( 'elementor/element/woocommerce-products/section_products_style/before_section_end', 'VamtamElementor\Widgets\ProductsBase\update_products_base_controls', 10, 2 );

function update_pagination_spacing_control( $controls_manager, $widget ) {
	$control_id   = 'pagination_spacing';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .navigation.vamtam-pagination-wrapper' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_pagination_border_color_control( $controls_manager, $widget ) {
	$control_id   = 'pagination_border_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .navigation.vamtam-pagination-wrapper .page-numbers' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_pagination_padding_control( $controls_manager, $widget ) {
	$control_id   = 'pagination_padding';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .navigation.vamtam-pagination-wrapper .page-numbers' => 'line-height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}}',
	];

	$widget->update_control( $control_id, $control_data );
}

function update_pagination_link_color_control( $controls_manager, $widget ) {
	$control_id   = 'pagination_link_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .navigation.vamtam-pagination-wrapper .page-numbers' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}


function update_pagination_link_bg_color_control( $controls_manager, $widget ) {
	$control_id   = 'pagination_link_bg_color';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .navigation.vamtam-pagination-wrapper .page-numbers' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_pagination_link_color_hover_control( $controls_manager, $widget ) {
	$control_id   = 'pagination_link_color_hover';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .navigation.vamtam-pagination-wrapper .page-numbers:hover' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_pagination_link_bg_color_hover_control( $controls_manager, $widget ) {
	$control_id   = 'pagination_link_bg_color_hover';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .navigation.vamtam-pagination-wrapper .page-numbers:hover' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_pagination_link_color_active_control( $controls_manager, $widget ) {
	$control_id   = 'pagination_link_color_active';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .navigation.vamtam-pagination-wrapper .page-numbers.current' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

function update_pagination_link_bg_color_active_control( $controls_manager, $widget ) {
	$control_id   = 'pagination_link_bg_color_active';
	$control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	$control_data['selectors'] = $control_data['selectors'] + [
		'{{WRAPPER}} .navigation.vamtam-pagination-wrapper .page-numbers.current' => reset( $control_data[ 'selectors' ] ),
	];

	$widget->update_control( $control_id, $control_data );
}

// Sadly there's no update_group_controls() function.
// We have to update each control in the group manually.
function update_pagination_typography_control_group( $controls_manager, $widget ) {
	$control_group = $controls_manager->get_control_groups( 'typography' );

	foreach ( $control_group->get_fields() as $key => $value ) {
	   $control_id   = 'pagination_typography_' . $key;
	   $control_data = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	   if ( is_wp_error( $control_data ) ) {
		   continue;
	   }

	   $selector_val = isset( $control_data['selector_value'] ) ? $control_data['selector_value'] : reset( $control_data[ 'selectors' ] );
	   $control_data['selectors'] = $control_data['selectors'] + [
			'{{WRAPPER}} .navigation.vamtam-pagination-wrapper' => $selector_val,
		];

	   // Update the control in the stack.
	   $widget->update_control( $control_id, $control_data );
   }
}

function update_pagination_base_controls( $widget, $args ) {
	$controls_manager = \Elementor\Plugin::instance()->controls_manager;
	update_pagination_spacing_control( $controls_manager, $widget );
	update_pagination_border_color_control( $controls_manager, $widget );
	update_pagination_padding_control( $controls_manager, $widget );
	update_pagination_link_color_control( $controls_manager, $widget );
	update_pagination_link_bg_color_control( $controls_manager, $widget );
	update_pagination_link_color_hover_control( $controls_manager, $widget );
	update_pagination_link_bg_color_hover_control( $controls_manager, $widget );
	update_pagination_link_color_active_control( $controls_manager, $widget );
	update_pagination_link_bg_color_active_control( $controls_manager, $widget );
	update_pagination_typography_control_group( $controls_manager, $widget );
}

// Products Archive Pagination section.
add_action( 'elementor/element/wc-archive-products/section_pagination_style/before_section_end', 'VamtamElementor\Widgets\ProductsBase\update_pagination_base_controls', 10, 2 );
