<?php
namespace VamtamElementor\Widgets\Slides;

// Extending the Slides widget.

// Repeater control.
function update_slides_repeater_control( $controls_manager, $widget ) {
	$control_id      = 'slides';
	$field_to_update = 'link';
	$control_data    = $controls_manager->get_control_from_stack( $widget->get_unique_name(), $control_id );

	if ( is_wp_error( $control_data ) ) {
		return;
	}

	// We can access and modify the repeater fields as an array directly
	$control_data['fields'][ $field_to_update ]['dynamic'] = [
		'active' => true,
	];

	$widget->update_control( $control_id, $control_data );
}

// Content - Slides section.
add_action( 'elementor/element/slides/section_slides/before_section_end', function( $widget, $args ) {
	$controls_manager = \Elementor\Plugin::instance()->controls_manager;
	update_slides_repeater_control( $controls_manager, $widget );
}, 10, 2 );

// Making the dynamic tags work with slide's link field (step 1 of 2).
add_action( 'elementor/frontend/widget/before_render', function ( \Elementor\Element_Base $element ) {
	if ( 'slides' === $element->get_name() ) {
		$settings = $element->get_settings_for_display();

		foreach ( $settings['slides'] as $key => $value) {
			// Tag the link elements that we'll need to update.
			$shouldTag = ! empty( $value['__dynamic__']['link'] ) && strpos( $value['link']['url'], '%23elementor-action') === 0;
			if ( $shouldTag ) {
				$element->add_render_attribute( "slide_link{$key}", 'href', "#vt-${key}#", true );
			}
		}
	}
} );

// Making the dynamic tags work with slide's link field (step 2 of 2).
add_action( 'elementor/widget/render_content', function( $content, $widget ) {
	if ( 'slides' === $widget->get_name() ) {
		$settings = $widget->get_settings_for_display();

		foreach ( $settings['slides'] as $key => $value) {
			// Replace the href attr of the tagged link elements with the proper action string.
			if ( ! empty( $value['__dynamic__']['link'] ) ) {
				$content = preg_replace(
					'/#vt-' . $key . '#[^"]*/',
					$value['link']['url'],
					$content
				);
			}
		}
	}

	return $content;
 }, 10, 2 );

