<?php

use \Elementor\Controls_Manager;
use \Elementor\Group_Control_Border;
use \Elementor\Group_Control_Box_Shadow;
use \Elementor\Group_Control_Typography;
use \Elementor\Scheme_Typography;
use \Elementor\Widget_Base;

if (!defined('ABSPATH')) {
    exit;
} // If this file is called directly, abort.

class Vamtam_Instagram_Feed extends Vamtam_Elementor_Widget {
	use \Vamtam_Widget_Helpers;

    public function get_name() {
        return 'vamtam-instafeed';
    }

    public function get_title() {
        return esc_html__('Vamtam Instagram Feed', 'vamtam-elementor-integration');
    }

    public function get_icon() {
        return 'fa fa-instagram';
    }

    public function get_categories() {
        return ['general'];
    }

    protected function _register_controls() {
        $this->start_controls_section(
            'vamtam_section_instafeed_settings_account',
            [
                'label' => esc_html__('Instagram Account Settings', 'vamtam-elementor-integration'),
            ]
        );

        $this->add_control(
            'vamtam_instafeed_access_token',
            [
                'label' => esc_html__('Access Token', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'default' => esc_html__('4507625822.ba4c844.2608ae40c33d40fe97bffdc9bed8c9c7', 'vamtam-elementor-integration'),
                'description' => '<a href="http://www.jetseotools.com/instagram-access-token/" class="vamtam-btn" target="_blank">Get Access Token</a>', 'vamtam-elementor-integration',
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'vamtam_section_instafeed_settings_content',
            [
                'label' => esc_html__('Feed Settings', 'vamtam-elementor-integration'),
            ]
        );

        $this->add_control(
            'vamtam_instafeed_sort_by',
            [
                'label' => esc_html__('Sort By', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SELECT,
                'default' => 'none',
                'options' => [
                    'none' => esc_html__('None', 'vamtam-elementor-integration'),
                    'most-recent' => esc_html__('Most Recent', 'vamtam-elementor-integration'),
                    'least-recent' => esc_html__('Least Recent', 'vamtam-elementor-integration'),
                    'most-liked' => esc_html__('Most Likes', 'vamtam-elementor-integration'),
                    'least-liked' => esc_html__('Least Likes', 'vamtam-elementor-integration'),
                    'most-commented' => esc_html__('Most Commented', 'vamtam-elementor-integration'),
                    'least-commented' => esc_html__('Least Commented', 'vamtam-elementor-integration'),
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_image_count',
            [
                'label' => esc_html__('Max Visible Images', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SLIDER,
                'default' => [
                    'size' => 12,
                ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_image_resolution',
            [
                'label' => esc_html__('Image Resolution', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SELECT,
                'default' => 'low_resolution',
                'options' => [
                    'thumbnail' => esc_html__('Thumbnail (150x150)', 'vamtam-elementor-integration'),
                    'low_resolution' => esc_html__('Low Res (306x306)', 'vamtam-elementor-integration'),
                    'standard_resolution' => esc_html__('Standard (612x612)', 'vamtam-elementor-integration'),
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_force_square',
            [
                'label' => esc_html__('Force Square Image?', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => '',
            ]
        );

        $this->add_responsive_control(
            'vamtam_instafeed_sq_image_size',
            [
                'label' => esc_html__('Image Dimension (px)', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SLIDER,
                'default' => [
                    'size' => 300,
                ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .vamtam-instafeed-square-img .vamtam-instafeed-item img' => 'width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}}; object-fit: cover;',
                ],
                'condition' => [
                    'vamtam_instafeed_force_square' => 'yes',
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'vamtam_section_instafeed_settings_general',
            [
                'label' => esc_html__('General Settings', 'vamtam-elementor-integration'),
            ]
        );

        $this->add_control(
            'vamtam_instafeed_layout',
            [
                'label' => esc_html__('Layout', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SELECT,
                'default' => 'overlay',
                'options' => [
                    'card' => esc_html__('Card', 'vamtam-elementor-integration'),
                    'overlay' => esc_html__('Overlay', 'vamtam-elementor-integration'),
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_card_style',
            [
                'label' => esc_html__('Card Style', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SELECT,
                'default' => 'outer',
                'options' => [
                    'inner' => esc_html__('Content Inner', 'vamtam-elementor-integration'),
                    'outer' => esc_html__('Content Outer', 'vamtam-elementor-integration'),
                ],
                'condition' => [
                    'vamtam_instafeed_layout' => 'card',
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_overlay_style',
            [
                'label' => esc_html__('Overlay Style', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SELECT,
                'default' => 'simple',
                'options' => [
                    'simple' => esc_html__('Simple', 'vamtam-elementor-integration'),
                    'basic' => esc_html__('Basic', 'vamtam-elementor-integration'),
                    'standard' => esc_html__('Standard', 'vamtam-elementor-integration'),
                ],
                'condition' => [
                    'vamtam_instafeed_layout' => 'overlay',
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_columns',
            [
                'label' => esc_html__('Number of Columns', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SELECT,
                'default' => 'vamtam-col-4',
                'options' => [
                    'vamtam-col-1' => esc_html__('Single Column', 'vamtam-elementor-integration'),
                    'vamtam-col-2' => esc_html__('Two Columns', 'vamtam-elementor-integration'),
                    'vamtam-col-3' => esc_html__('Three Columns', 'vamtam-elementor-integration'),
                    'vamtam-col-4' => esc_html__('Four Columns', 'vamtam-elementor-integration'),
                    'vamtam-col-5' => esc_html__('Five Columns', 'vamtam-elementor-integration'),
                    'vamtam-col-6' => esc_html__('Six Columns', 'vamtam-elementor-integration'),
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_pagination_heading',
            [
                'label' => __('Pagination', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::HEADING,
            ]
        );

        $this->add_control(
            'vamtam_instafeed_pagination',
            [
                'label' => esc_html__('Enable Load More?', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => '',
            ]
        );

        $this->add_control(
			'loadmore_text',
			[
				'label'     => __( 'Label', 'vamtam-elementor-integration' ),
				'type'      => Controls_Manager::TEXT,
                'default'   => __( 'Load More', 'vamtam-elementor-integration' ),
                'condition' => [
                    'vamtam_instafeed_pagination' => 'yes'
                ]
			]
		);

        $this->add_control(
            'vamtam_instafeed_caption_heading',
            [
                'label' => __('Link & Content', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::HEADING,
            ]
        );

        $this->add_control(
            'vamtam_instafeed_caption',
            [
                'label' => esc_html__('Display Caption', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'vamtam_instafeed_likes',
            [
                'label' => esc_html__('Display Like', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'vamtam_instafeed_comments',
            [
                'label' => esc_html__('Display Comments', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'vamtam_instafeed_date',
            [
                'label' => esc_html__('Display Date', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'vamtam_instafeed_link',
            [
                'label' => esc_html__('Enable Link', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'vamtam_instafeed_link_target',
            [
                'label' => esc_html__('Open in new window?', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
                'condition' => [
                    'vamtam_instafeed_link' => 'yes',
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'vamtam_section_instafeed_styles_general',
            [
                'label' => esc_html__('Instagram Feed Styles', 'vamtam-elementor-integration'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_responsive_control(
            'vamtam_instafeed_spacing',
            [
                'label' => esc_html__('Padding Between Images', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%', 'em'],
                'selectors' => [
                    '{{WRAPPER}} .vamtam-instafeed-item-inner' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'vamtam_instafeed_box_border',
                'label' => esc_html__('Border', 'vamtam-elementor-integration'),
                'selector' => '{{WRAPPER}} .vamtam-instafeed-item-inner',
            ]
        );

        $this->add_control(
            'vamtam_instafeed_box_border_radius',
            [
                'label' => esc_html__('Border Radius', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::DIMENSIONS,
                'selectors' => [
                    '{{WRAPPER}} .vamtam-instafeed-item-inner' => 'border-radius: {{TOP}}px {{RIGHT}}px {{BOTTOM}}px {{LEFT}}px;',
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'vamtam_section_instafeed_styles_content',
            [
                'label' => esc_html__('Color &amp; Typography', 'vamtam-elementor-integration'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_control(
            'vamtam_instafeed_overlay_color',
            [
                'label' => esc_html__('Overlay Color', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::COLOR,
                'default' => 'rgba(137,12,255,0.75)',
                'selectors' => [
                    '{{WRAPPER}} .vamtam-instafeed-caption' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_like_comments_heading',
            [
                'label' => __('Icon Styles', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::HEADING,
            ]
        );

        $this->add_control(
            'vamtam_instafeed_icon_color',
            [
                'label' => esc_html__('Icon Color', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::COLOR,
                'default' => '#ffffff',
                'selectors' => [
                    '{{WRAPPER}} .vamtam-instafeed-caption i' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_caption_style_heading',
            [
                'label' => __('Caption Styles', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::HEADING,
            ]
        );

        $this->add_control(
            'vamtam_instafeed_caption_color',
            [
                'label' => esc_html__('Caption Color', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::COLOR,
                'default' => '#ffffff',
                'selectors' => [
                    '{{WRAPPER}} .vamtam-instafeed-caption' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'vamtam_instafeed_caption_typography',
                'scheme' => Scheme_Typography::TYPOGRAPHY_2,
                'selector' => '{{WRAPPER}} .vamtam-instafeed-caption',
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'vamtam_section_load_more_btn',
            [
                'label' => __('Load More Button Style', 'vamtam-elementor-integration'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_responsive_control(
            'vamtam_instafeed_load_more_btn_padding',
            [
                'label' => esc_html__('Padding', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', 'em', '%'],
                'selectors' => [
                    '{{WRAPPER}} .vamtam-load-more-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'vamtam_instafeed_load_more_btn_margin',
            [
                'label' => esc_html__('Margin', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', 'em', '%'],
                'selectors' => [
                    '{{WRAPPER}} .vamtam-load-more-button' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'vamtam_instafeed_load_more_btn_typography',
                'selector' => '{{WRAPPER}} .vamtam-load-more-button',
            ]
        );

        $this->start_controls_tabs('vamtam_instafeed_load_more_btn_tabs');

        // Normal State Tab
        $this->start_controls_tab('vamtam_instafeed_load_more_btn_normal', ['label' => esc_html__('Normal', 'vamtam-elementor-integration')]);

        $this->add_control(
            'vamtam_instafeed_load_more_btn_normal_text_color',
            [
                'label' => esc_html__('Text Color', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::COLOR,
                'default' => '#fff',
                'selectors' => [
                    '{{WRAPPER}} .vamtam-load-more-button' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'vamtam_cta_btn_normal_bg_color',
            [
                'label' => esc_html__('Background Color', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::COLOR,
                'default' => '#29d8d8',
                'selectors' => [
                    '{{WRAPPER}} .vamtam-load-more-button' => 'background: {{VALUE}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'vamtam_instafeed_load_more_btn_normal_border',
                'label' => esc_html__('Border', 'vamtam-elementor-integration'),
                'selector' => '{{WRAPPER}} .vamtam-load-more-button',
            ]
        );

        $this->add_control(
            'vamtam_instafeed_load_more_btn_border_radius',
            [
                'label' => esc_html__('Border Radius', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .vamtam-load-more-button' => 'border-radius: {{SIZE}}px;',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'vamtam_instafeed_load_more_btn_shadow',
                'selector' => '{{WRAPPER}} .vamtam-load-more-button',
                'separator' => 'before',
            ]
        );

        $this->end_controls_tab();

        // Hover State Tab
        $this->start_controls_tab('vamtam_instafeed_load_more_btn_hover', ['label' => esc_html__('Hover', 'vamtam-elementor-integration')]);

        $this->add_control(
            'vamtam_instafeed_load_more_btn_hover_text_color',
            [
                'label' => esc_html__('Text Color', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::COLOR,
                'default' => '#fff',
                'selectors' => [
                    '{{WRAPPER}} .vamtam-load-more-button:hover' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_load_more_btn_hover_bg_color',
            [
                'label' => esc_html__('Background Color', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::COLOR,
                'default' => '#27bdbd',
                'selectors' => [
                    '{{WRAPPER}} .vamtam-load-more-button:hover' => 'background: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'vamtam_instafeed_load_more_btn_hover_border_color',
            [
                'label' => esc_html__('Border Color', 'vamtam-elementor-integration'),
                'type' => Controls_Manager::COLOR,
                'default' => '',
                'selectors' => [
                    '{{WRAPPER}} .vamtam-load-more-button:hover' => 'border-color: {{VALUE}};',
                ],
            ]

        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'vamtam_instafeed_load_more_btn_hover_shadow',
                'selector' => '{{WRAPPER}} .vamtam-load-more-button:hover',
                'separator' => 'before',
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();
    }

    protected function render() {
        $settings = $this->get_settings_for_display();
        // print_r($settings);
        $classes = implode(' ', [
            $settings['vamtam_instafeed_columns'],
            ($settings['vamtam_instafeed_force_square'] == 'yes') ? "vamtam-instafeed-square-img" : "",
            'vamtam-instafeed-' . $settings['vamtam_instafeed_layout'],
            'vamtam-instafeed-' . $settings['vamtam_instafeed_layout'] . '-' . $settings["vamtam_instafeed_{$settings['vamtam_instafeed_layout']}_style"]
        ]);
        $settings_var = [
            'vamtam_instafeed_access_token' => $settings['vamtam_instafeed_access_token'],
            'vamtam_instafeed_image_count' => $settings['vamtam_instafeed_image_count'],
            'vamtam_instafeed_sort_by' => $settings['vamtam_instafeed_sort_by'],
            'vamtam_instafeed_image_resolution' => $settings['vamtam_instafeed_image_resolution'],
            'vamtam_instafeed_layout' => $settings['vamtam_instafeed_layout'],
            'vamtam_instafeed_card_style' => $settings['vamtam_instafeed_card_style'],
            'vamtam_instafeed_overlay_style' => $settings['vamtam_instafeed_overlay_style'],
            'vamtam_instafeed_date' => $settings['vamtam_instafeed_date'],
            'vamtam_instafeed_likes' => $settings['vamtam_instafeed_likes'],
            'vamtam_instafeed_comments' => $settings['vamtam_instafeed_comments'],
            'vamtam_instafeed_caption' => $settings['vamtam_instafeed_caption'],
            'vamtam_instafeed_link' => $settings['vamtam_instafeed_link'],
            'vamtam_instafeed_link_target' => $settings['vamtam_instafeed_link_target']
        ];

        echo '<div id="vamtam-instafeed-' . esc_attr($this->get_id()) . '" class="vamtam-instafeed ' . $classes . '">
            ' . $this->instafeed_render_items() . '
        </div>
        <div class="clearfix"></div>';

        if (($settings['vamtam_instafeed_pagination'] == 'yes')) {
            echo '<div class="vamtam-load-more-button-wrap">
                <button class="vamtam-load-more-button" id="vamtam-load-more-btn-' . $this->get_id() . '" data-settings="' . http_build_query($settings_var) . '" data-page="1">
                    <div class="vamtam-btn-loader button__loader"></div>
                    <span>'.$settings['loadmore_text'].'</span>
                </button>
            </div>';
        }
        echo '</div>';
	}
}
