<?php

trait Vamtam_Widget_Helpers {

	public function register_widget_ajax_actions() {
		// Vamtam Instagram Feed widget.
		add_action('wp_ajax_instafeed_load_more', [$this, 'instafeed_render_items']);
		add_action('wp_ajax_nopriv_instafeed_load_more', [$this, 'instafeed_render_items']);
	}

	function instafeed_render_items() {
        // check if ajax request
        if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'instafeed_load_more') {
            // check ajax referer
            check_ajax_referer('instafeed_load_more', 'security');
            // init vars
            $page = $_REQUEST['page'];
            parse_str($_REQUEST['settings'], $settings);
        } else {
            // init vars
            $page = 0;
            $settings = $this->get_settings();
        }

        $key = 'vamtam_instafeed_' . str_replace('.', '_', $settings['vamtam_instafeed_access_token']);
        $html = '';

        if (get_transient($key) === false) {
            $instagram_data = wp_remote_retrieve_body(wp_remote_get('https://api.instagram.com/v1/users/self/media/recent/?access_token=' . $settings['vamtam_instafeed_access_token']));
            set_transient($key, $instagram_data, 1800);
        } else {
            $instagram_data = get_transient($key);
        }

        $instagram_data = json_decode($instagram_data, true);

        if (empty($instagram_data['data'])) {
            return;
        }

        if (empty($settings['vamtam_instafeed_image_count']['size'])) {
            return;
        }

        switch ($settings['vamtam_instafeed_sort_by']) {
            case 'most-recent':
                sort($instagram_data['data']);
                break;

            case 'least-recent':
                rsort($instagram_data['data']);
                break;

            case 'most-liked':
                usort($instagram_data['data'], function ($a, $b) {
                    return $a['likes']['count'] <= $b['likes']['count'];
                });
                break;

            case 'least-liked':
                usort($instagram_data['data'], function ($a, $b) {
                    return $a['likes']['count'] >= $b['likes']['count'];
                });
                break;

            case 'most-commented':
                usort($instagram_data['data'], function ($a, $b) {
                    return $a['comments']['count'] <= $b['comments']['count'];
                });
                break;

            case 'least-commented':
                usort($instagram_data['data'], function ($a, $b) {
                    return $a['comments']['count'] >= $b['comments']['count'];
                });
                break;
        }

        if ($items = $instagram_data['data']) {
            $items = array_splice($items, ($page * $settings['vamtam_instafeed_image_count']['size']), $settings['vamtam_instafeed_image_count']['size']);

            foreach ($items as $item) {
                if($settings['vamtam_instafeed_layout'] == 'overlay') {
                    $html .= '<a href="' . $item['link'] . '" target="' . ($settings['vamtam_instafeed_link_target'] ? '_blank' : '_self') . '" class="vamtam-instafeed-item">
                        <div class="vamtam-instafeed-item-inner">
                            <img class="vamtam-instafeed-img" src="' . $item['images'][$settings['vamtam_instafeed_image_resolution']]['url'] . '">

                            <div class="vamtam-instafeed-caption">
                                <div class="vamtam-instafeed-caption-inner">';
                                    if ($settings['vamtam_instafeed_overlay_style'] == 'simple' || $settings['vamtam_instafeed_overlay_style'] == 'standard') {
                                        $html .= '<div class="vamtam-instafeed-icon">
                                            <i class="fab fa-instagram" aria-hidden="true"></i>
                                        </div>';
                                    } else if ($settings['vamtam_instafeed_overlay_style'] == 'basic') {
                                        if ($settings['vamtam_instafeed_caption'] && !empty($item['caption']['text'])) {
                                            $html .= '<p class="vamtam-instafeed-caption-text">' . substr($item['caption']['text'], 0, 60) . '...</p>';
                                        }
                                    }

                                    $html .= '<div class="vamtam-instafeed-meta">';
                                        if ($settings['vamtam_instafeed_overlay_style'] == 'basic' && $settings['vamtam_instafeed_date']) {
                                            $html .= '<span class="vamtam-instafeed-post-time"><i class="far fa-clock" aria-hidden="true"></i> ' . date("d M Y", $item['created_time']) . '</span>';
                                        }
                                        if ($settings['vamtam_instafeed_overlay_style'] == 'standard') {
                                            if ($settings['vamtam_instafeed_caption'] && !empty($item['caption']['text'])) {
                                                $html .= '<p class="vamtam-instafeed-caption-text">' . substr($item['caption']['text'], 0, 60) . '...</p>';
                                            }
                                        }
                                        if ($settings['vamtam_instafeed_likes']) {
                                            $html .= '<span class="vamtam-instafeed-post-likes"><i class="far fa-heart" aria-hidden="true"></i> ' . $item['likes']['count'] . '</span>';
                                        }
                                        if ($settings['vamtam_instafeed_comments']) {
                                            $html .= '<span class="vamtam-instafeed-post-comments"><i class="far fa-comments" aria-hidden="true"></i> ' . $item['comments']['count'] . '</span>';
                                        }
                                    $html .= '</div>';
                                $html .= '</div>
                            </div>
                        </div>
                    </a>';
                } else {
                    $html .= '<div class="vamtam-instafeed-item">
                        <div class="vamtam-instafeed-item-inner">
                            <header class="vamtam-instafeed-item-header clearfix">
                                <div class="vamtam-instafeed-item-user clearfix">
                                    <a href="//www.instagram.com/' . $item['user']['username'] . '"><img src="' . $item['user']['profile_picture'] . '" alt="' . $item['user']['username'] . '" class="vamtam-instafeed-avatar"></a>
                                    <a href="//www.instagram.com/' . $item['user']['username'] . '"><p class="vamtam-instafeed-username">' . $item['user']['full_name'] . '</p></a>
                                </div>
                                <span class="vamtam-instafeed-icon"><i class="fab fa-instagram" aria-hidden="true"></i></span>';

                                if ($settings['vamtam_instafeed_date'] && $settings['vamtam_instafeed_card_style'] == 'outer') {
                                    $html .= '<span class="vamtam-instafeed-post-time"><i class="far fa-clock" aria-hidden="true"></i> ' . date("d M Y", $item['created_time']) . '</span>';
                                }
                            $html .= '</header>
                            <a href="' . $item['link'] . '" target="' . ($settings['vamtam_instafeed_link_target'] ? '_blank' : '_self') . '" class="vamtam-instafeed-item-content">
                                <img class="vamtam-instafeed-img" src="' . $item['images'][$settings['vamtam_instafeed_image_resolution']]['url'] . '">';

                                if ($settings['vamtam_instafeed_card_style'] == 'inner' && $settings['vamtam_instafeed_caption'] && !empty($item['caption']['text'])) {
                                    $html .= '<div class="vamtam-instafeed-caption">
                                        <div class="vamtam-instafeed-caption-inner">
                                            <div class="vamtam-instafeed-meta">
                                                <p class="vamtam-instafeed-caption-text">' . substr($item['caption']['text'], 0, 60) . '...</p>
                                            </div>
                                        </div>
                                    </div>';
                                }
                            $html .= '</a>
                            <footer class="vamtam-instafeed-item-footer">
                                <div class="clearfix">';
                                    if ($settings['vamtam_instafeed_likes']) {
                                        $html .= '<span class="vamtam-instafeed-post-likes"><i class="far fa-heart" aria-hidden="true"></i> ' . $item['likes']['count'] . '</span>';
                                    }
                                    if ($settings['vamtam_instafeed_comments']) {
                                        $html .= '<span class="vamtam-instafeed-post-comments"><i class="far fa-comments" aria-hidden="true"></i> ' . $item['comments']['count'] . '</span>';
                                    }
                                    if ($settings['vamtam_instafeed_card_style'] == 'inner' && $settings['vamtam_instafeed_date']) {
                                        $html .= '<span class="vamtam-instafeed-post-time"><i class="far fa-clock" aria-hidden="true"></i> ' . date("d M Y", $item['created_time']) . '</span>';
                                    }
                                $html .= '</div>';

                                if ($settings['vamtam_instafeed_card_style'] == 'outer' && $settings['vamtam_instafeed_caption'] && !empty($item['caption']['text'])) {
                                    $html .= '<p class="vamtam-instafeed-caption-text">' . substr($item['caption']['text'], 0, 60) . '...</p>';
                                }
                            $html .= '</footer>
                        </div>
                    </div>';
                }
            }
        }

        if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'instafeed_load_more') {
            wp_send_json([
                'num_pages' => ceil(count($instagram_data['data']) / $settings['vamtam_instafeed_image_count']['size']),
                'html' => $html
            ]);
        }

        return $html;
	}
}
