<?php

/**
 * Plugin Name: VamTam Elementor Integration
 * Plugin URI: http://vamtam.com
 * Description: Extend the Elementor plugin with widgets and template builder.
 * Version: 1.0.0
 * Author: VamTam
 * Author URI: http://vamtam.com
 */

// Widget Helpes TODO: Maybe use an intermidiate initializer class.
require_once plugin_dir_path( __FILE__ ) . 'includes/widgets/vamtam/vamtam-widget-helpers.php';

class VamtamElementorIntregration {

	use \Vamtam_Widget_Helpers;

	const PLUGIN_VERSION            = '1.0.0';
    const MINIMUM_ELEMENTOR_VERSION = '2.0.0';
    const MINIMUM_PHP_VERSION       = '7.0';

    private static $_instance = null;

    public static function instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    public function __construct() {
        // Abort early if not valid Vamtam theme.
        if ( ! self::is_valid_theme() ) {
            add_action( 'admin_notices', [ $this, 'admin_notice_invalid_theme' ] );
            return;
        }

        add_action( 'init', [ $this, 'i18n' ] );
        add_action( 'plugins_loaded', [ $this, 'init' ] );
    }

    public function init() {
        // Check if Elementor installed and activated
        if ( ! did_action( 'elementor/loaded' ) ) {
            add_action( 'admin_notices', [ $this, 'admin_notice_missing_main_plugin' ] );
            return;
        }

        // Check for required Elementor version
        if ( ! version_compare( ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=' ) ) {
            add_action( 'admin_notices', [ $this, 'admin_notice_minimum_elementor_version' ] );
            return;
        }

        // Check for required PHP version
        if ( version_compare( PHP_VERSION, self::MINIMUM_PHP_VERSION, '<' ) ) {
            add_action( 'admin_notices', [ $this, 'admin_notice_minimum_php_version' ] );
            return;
        }

        // All checks passed, load the plugin.
        $this->load_plugin();
    }

    public function i18n() {
        load_plugin_textdomain( 'vamtam-elementor-integration' );
    }

    public function admin_notice_invalid_theme() {
        if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

        $message = sprintf(
            /* translators: 1: Plugin name 2: Elementor */
            esc_html__( '"%1$s" requires an Elementor-based VamTam theme to be activate in order to work.', 'vamtam-elementor-integration' ),
            '<strong>' . esc_html__( 'Vamtam Elementor Integration', 'vamtam-elementor-integration' ) . '</strong>'
        );

        printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
    }

    public function admin_notice_missing_main_plugin() {
        if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

        $message = sprintf(
            /* translators: 1: Plugin name 2: Elementor */
            esc_html__( '"%1$s" requires "%2$s" to be installed and activated.', 'vamtam-elementor-integration' ),
            '<strong>' . esc_html__( 'Vamtam Elementor Integration', 'vamtam-elementor-integration' ) . '</strong>',
            '<strong>' . esc_html__( 'Elementor', 'vamtam-elementor-integration' ) . '</strong>'
        );

        printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
    }

    public function admin_notice_minimum_elementor_version() {
        if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

        $message = sprintf(
            /* translators: 1: Plugin name 2: Elementor 3: Required Elementor version */
            esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'vamtam-elementor-integration' ),
            '<strong>' . esc_html__( 'Vamtam Elementor Integration', 'vamtam-elementor-integration' ) . '</strong>',
            '<strong>' . esc_html__( 'Elementor', 'vamtam-elementor-integration' ) . '</strong>',
             self::MINIMUM_ELEMENTOR_VERSION
        );

        printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
    }

    public function load_plugin() {
        if ( ! class_exists( 'Vamtam_Updates_2' ) ) {
            require 'vamtam-updates/class-vamtam-updates.php';
        }

        new Vamtam_Updates_2( __FILE__ );

        if ( ! defined( 'VAMTAM_ELEMENTOR_INT_URL' ) ) {
            define( 'VAMTAM_ELEMENTOR_INT_URL', plugin_dir_url( __FILE__ ) );
        }
        if ( ! defined( 'VAMTAM_ELEMENTOR_INT_DIR' ) ) {
            define( 'VAMTAM_ELEMENTOR_INT_DIR', plugin_dir_path( __FILE__ ) );
        }

        $this->add_actions();
    }

    public function add_actions() {
        // Customizer.
        add_action( 'customize_register', [ __CLASS__, 'customize_register' ], 20 );

        // Used to print the custom JS.
        add_action( 'wp_head', [ __CLASS__, 'late_wp_head' ], 100 );
        add_action( 'wp_body_open', [ __CLASS__, 'early_wp_body_open' ], 5 );
        add_action( 'wp_footer', [ __CLASS__, 'late_wp_footer' ], 100 );
        add_action( 'wp_footer', [ __CLASS__, 'late_wp_footer' ], 100 );

		// Elementor actions.
        add_action( 'elementor/widgets/widgets_registered', [ $this, 'register_widgets' ] );
        add_action( 'elementor/controls/controls_registered', [ $this, 'register_controls' ] );
        add_action( 'elementor/frontend/after_register_scripts', [ $this, 'enqueue_frontend_scripts' ] );
		add_action( 'elementor/frontend/after_enqueue_styles', [ $this, 'enqueue_frontend_styles' ] );
		$this->register_widget_ajax_actions();

		// Elementor includes.
		$this->includes();

		// TODO: To be removed when https://github.com/elementor/elementor/issues/10649 is fixed by Elementor.
		add_action( 'elementor/frontend/after_enqueue_styles',[ $this, 'change_elementor_frontend_css' ] );
    }

    public function includes() {
        // All Elementor-related hooks.
        require_once VAMTAM_ELEMENTOR_INT_DIR . 'includes/vamtam-elementor-hooks.php';

        // Kits Overrides.
        foreach( glob( VAMTAM_ELEMENTOR_INT_DIR . 'includes/kits/**/*.php' ) as $kit ) {
            require_once $kit;
        }

        // Widget Overrides.
        foreach( glob( VAMTAM_ELEMENTOR_INT_DIR . 'includes/widgets/*.php' ) as $widget ) {
            require_once $widget;
        }
	}

    public static function is_valid_theme() {
        $theme = wp_get_theme();
        $is_valid = ! empty( $theme ) && $theme->get('Author')  === 'VamTam';
        return $is_valid;
    }

    public function enqueue_frontend_scripts() {
        $suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

        // Vamtam Instagram Feed js.
        wp_enqueue_script(
            'vamtam-instagram-feed',
            VAMTAM_ELEMENTOR_INT_URL . '/assets/js/widgets/vamtam-instagram-feed' . $suffix . '.js',
            [
                'jquery',
            ],
            self::PLUGIN_VERSION,
            true // in footer
		);

		wp_localize_script( 'vamtam-instagram-feed', 'vif', [
			'nonce' => wp_create_nonce( 'instafeed_load_more' )
		] );
    }

    public function enqueue_frontend_styles() {
        $suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

        // Vamtam Instagram Feed css.
        wp_enqueue_style(
            'vamtam-instagram-feed',
            VAMTAM_ELEMENTOR_INT_URL . '/assets/css/widgets/vamtam-instagram-feed' . $suffix . '.css',
            [],
            self::PLUGIN_VERSION
        );
    }

    public static function enqueue_scripts() {
        // Enqueue JS for Elementor (frontend).
        wp_enqueue_script(
            'vamtam-elementor-integration',
            VAMTAM_ELEMENTOR_INT_URL . '/assets/js/vamtam-elementor-frontend.js',
            [
                'elementor-frontend', // dependency
            ],
            self::PLUGIN_VERSION,
            true //in footer
        );
    }

    public static function enqueue_editor_scripts() {
        // Enqueue CSS for Elementor editor.
        wp_enqueue_style( 'vamtam-elementor', VAMTAM_ELEMENTOR_INT_URL . '/assets/css/vamtam-elementor.css', [], self::PLUGIN_VERSION );

        // Export theme options (css vars) to Elementor editor.
        VamtamEnqueues::print_theme_options();

        // Enqueue JS for Elementor editor.
        wp_enqueue_script( 'vamtam-elementor', VAMTAM_ELEMENTOR_INT_URL . '/assets/js/vamtam-elementor.js', [], self::PLUGIN_VERSION, true );

        global $vamtam_theme;

        wp_localize_script(
            'vamtam-elementor-integration', 'VamtamColorPickerStrings', array(
                'accents'             => isset( $vamtam_theme[ 'accent-color' ] ) ? array_filter( $vamtam_theme[ 'accent-color' ], function( $key ) {
                    return is_numeric( $key );
                }, ARRAY_FILTER_USE_KEY ) : [],
            )
        );
    }

    public static function late_wp_head() {
        $additional_js = get_option( 'vamtam_additional_js' );

        if ( $additional_js && isset( $additional_js['head'] ) ) {
            echo '<script>' . $additional_js['head'] . '</script>';
        }
    }

    public static function early_wp_body_open() {
        $additional_js = get_option( 'vamtam_additional_js' );

        if ( $additional_js && isset( $additional_js['body'] ) ) {
            echo '<script>' . $additional_js['body'] . '</script>';
        }
    }

    public static function late_wp_footer() {
        $additional_js = get_option( 'vamtam_additional_js' );

        if ( $additional_js && isset( $additional_js['footer'] ) ) {
            echo '<script>' . $additional_js['footer'] . '</script>';
        }
    }

    /**
     * Check if standalone Elementor Pro is active
     *
     * @return boolean
     */
    private static function is_elementor_pro_active() {
        include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        return is_plugin_active( 'elementor-pro/elementor-pro.php' );
    }

    /**
     * Check if it is the elementor pro plugin activation screen
     *
     * @return boolean
     */
    public static function is_elementor_pro_standalone_plugin_activation_screen() {
        if ( is_admin() && isset( $_GET['plugin'] ) && $_GET['plugin'] == 'elementor-pro/elementor-pro.php' ) {
            return true;
        }

        return false;
    }

    public function register_widgets() {
        // Base Vamtam Widget
        require_once VAMTAM_ELEMENTOR_INT_DIR . 'includes/widgets/vamtam/base/vamtam-widget.php';

        // Vamtam Widgets.
        foreach( glob( VAMTAM_ELEMENTOR_INT_DIR . 'includes/widgets/vamtam/*.php' ) as $widget ) {
            require_once $widget;
        }

        $widgets_manager = \Elementor\Plugin::instance()->widgets_manager;
        $widgets_manager->register_widget_type( new Vamtam_Instagram_Feed );
    }

    public function register_controls() {
        foreach( glob( VAMTAM_ELEMENTOR_INT_DIR . 'includes/controls/*.php' ) as $control ) {
            require_once $control;
        }

        $controls_manager = \Elementor\Plugin::$instance->controls_manager;
        $controls_manager->register_control( 'vamtam-color', new Vamtam_Control_Color() );
	}

	/*
		TODO: To be removed when https://github.com/elementor/elementor/issues/10649 is fixed by Elementor.
		Patch by: https://github.com/elementor/elementor/issues/10649#issuecomment-604813342
	*/
	public function change_elementor_frontend_css() {
		if ( ! defined( 'ELEMENTOR_VERSION' ) ) {
			return;
		}

		global $wp_styles;

		$path = $_SERVER["DOCUMENT_ROOT"] . parse_url( $wp_styles->registered[ 'elementor-frontend' ]->src )["path"];
		$css  = file_get_contents( $path );
		$css  = str_replace( ".elementor-widget-heading .elementor-heading-title{padding:0;margin:0;line-height:1}", '.elementor-widget-heading .elementor-heading-title{padding:0;margin:0;}', $css, $count );

		if ( $count ) {
			file_put_contents( $path, $css );
		}
	}

    public static function customize_register( WP_Customize_Manager $wp_customize  ) {
        $wp_customize->add_section( 'vamtam_additional_js' , [
            'title'      => esc_html__( 'Additional JS', 'vamtam-elementor-integration' ),
            'priority'   => 200,
        ] );

        $wp_customize->add_setting(
            'vamtam_additional_js[head]',
            [
                'type'    => 'option',
                'default' => '',
            ]
        );

        $wp_customize->add_setting(
            'vamtam_additional_js[body]',
            [
                'type'    => 'option',
                'default' => '',
            ]
        );

        $wp_customize->add_setting(
            'vamtam_additional_js[footer]',
            [
                'type'    => 'option',
                'default' => '',
            ]
        );

        $wp_customize->add_control(
            new WP_Customize_Code_Editor_Control(
                $wp_customize,
                'vamtam_additional_js[head]',
                [
                    'label'      => __( 'Before </head>', 'vamtam-elementor-integration' ), // control labels are escaped, using esc_html here will encode it as &amp;lt;
                    'section'    => 'vamtam_additional_js',
                    'settings'   => 'vamtam_additional_js[head]',
                    'code_type'  => 'javascript',
                ]
            )
        );

        $wp_customize->add_control(
            new WP_Customize_Code_Editor_Control(
                $wp_customize,
                'vamtam_additional_js[body]',
                [
                    'label'      => __( 'After <body>', 'vamtam-elementor-integration' ), // control labels are escaped, using esc_html here will encode it as &amp;lt;
                    'section'    => 'vamtam_additional_js',
                    'settings'   => 'vamtam_additional_js[body]',
                    'code_type'  => 'javascript',
                ]
            )
        );

        $wp_customize->add_control(
            new WP_Customize_Code_Editor_Control(
                $wp_customize,
                'vamtam_additional_js[footer]',
                [
                    'label'      => __( 'Before </body>', 'vamtam-elementor-integration' ), // control labels are escaped, using esc_html here will encode it as &amp;lt;
                    'section'    => 'vamtam_additional_js',
                    'settings'   => 'vamtam_additional_js[footer]',
                    'code_type'  => 'javascript',
                ]
            )
        );
    }
}

VamtamElementorIntregration::instance();
